/*
 * busywidget.h - cool animating widget
 * Copyright (C) 2001, 2002  Justin Karneges
 *                           Hideaki Omuro
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef BUSYWIDGET_H
#define BUSYWIDGET_H

#include<qwidget.h>
#include<qpixmap.h>

class CColor;

// panel class
class CPanel
{
public:
	int angle;
	int height;
	bool spinning;
	int alpha;

	CPanel(int height = 1);
	int GetModHeight();
	int GetShade();
	void Spin(int n);

	void SetAngle(int _angle)	{ angle = _angle % 1024; }
	void SetHeight(int _height)	{ height = _height; }
	int GetAngle()				{ return angle % 1024; }
	int GetHeight()				{ return height; }
	int GetModOffset()			{ return (height - GetModHeight()) / 2; }
};

class BusyWidget : public QWidget
{
	Q_OBJECT
public:
	BusyWidget(QWidget *parent=0, const char *name=0);

	void start();
	void stop();
	bool isActive() { return v_isActive; }

protected:
	void paintEvent(QPaintEvent *);
	void resizeEvent(QResizeEvent *);

private:
	bool v_isActive;
	int frame;
	int at;
	QPixmap pix;
	QTimer *t;

	// data
	CPanel panel[5];
	int  pcountdown;
	int  ocountdown;

	void render();
	void renderPixmap();

private slots:
	void animate();
};

// color class
class CColor
{
public:
	int m_clr;

	CColor(int _r, int _g, int _b)
	{
		SetColor(_r, _g, _b);
	}
	CColor(int _clr)
	{
		SetColor(_clr);
	}
	inline void SetColor(int _r, int _g, int _b)
	{
		SetColor((_r << 16) + (_g << 8) + _b);
	}
	inline void SetColor(int _clr)
	{
		m_clr = _clr;
	}
	inline short GetR()
	{
		return m_clr >> 16;
	}
	inline short GetG()
	{
		return (m_clr >> 8) & 255;
	}
	inline short GetB()
	{
		return m_clr & 255;
	}
	CColor Alpha(CColor clr, int alpha = 256);
};

#endif
